package com.jkcredit.invoice.service.lowerService.impl;
import cn.com.taiji.sdk.model.comm.protocol.tts.card.server.*;
import cn.com.taiji.sdk.model.comm.protocol.tts.invoice.server.*;
import cn.com.taiji.sdk.model.comm.protocol.tts.trade.service.CardTradeModel;
import cn.com.taiji.sdk.model.comm.protocol.tts.trade.service.CardTradeRequest;
import cn.com.taiji.sdk.model.comm.protocol.tts.trade.service.CardTradeResponse;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.Binvoce.SelfCarApplMapper;
import com.jkcredit.invoice.mapper.Binvoce.SelfCarInvoiceMapper;
import com.jkcredit.invoice.mapper.calculateInfor.SelfCarCalculateInforMapper;
import com.jkcredit.invoice.mapper.customer.CustomerCarRecMapper;
import com.jkcredit.invoice.mapper.customer.CustomerMapper;
import com.jkcredit.invoice.mapper.customer.CustomerRecMapper;
import com.jkcredit.invoice.mapper.waybill.SellCarTradeMapper;
import com.jkcredit.invoice.model.entity.Calculate.SelfCarCalculateInfor;
import com.jkcredit.invoice.model.entity.customer.Customer;
import com.jkcredit.invoice.model.entity.customer.CustomerCarRec;
import com.jkcredit.invoice.model.entity.customer.CustomerRec;
import com.jkcredit.invoice.model.entity.invoice.SelfCarAppl;
import com.jkcredit.invoice.model.entity.invoice.SelfCarInvoice;
import com.jkcredit.invoice.model.entity.waybill.SelfCarTrade;
import com.jkcredit.invoice.service.customer.CustomerService;
import com.jkcredit.invoice.service.lowerService.SelfCarServiceL;
import com.jkcredit.invoice.service.lowerService.vo.*;
import com.jkcredit.invoice.service.upService.SelfCarInterface;
import com.jkcredit.invoice.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
@Service
@Slf4j
public class SelfCarServiceLImpl implements SelfCarServiceL {


    
    
    @Autowired
    SelfCarInterface selfCarInterface;
    @Autowired
    CustomerCarRecMapper customerCarRecMapper;
    @Autowired
    CustomerMapper customerMapper;
    @Autowired
    CustomerRecMapper customerRecMapper;
    @Autowired
    SellCarTradeMapper sellCarTradeMapper;
    @Autowired
    SelfCarApplMapper selfCarApplMapper;
    @Autowired
    SelfCarInvoiceMapper selfCarInvoiceMapper;
    @Autowired
    SelfCarCalculateInforMapper calculateInforMapper;
    @Autowired
    CustomerService customerService;
    @Override
    @Transactional
    public RespR<List<CardTradeModel>> getTradeList(TradeRequestVo tradeRequestVo) {
        log.info("获取交易开始SelfCarServiceLImpl.getTradeList{}",tradeRequestVo);
        Customer customer = customerMapper.selectByCustomerName(tradeRequestVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            log.info("获取交易失败SelfCarServiceLImpl.getTradeList{},msg{}",tradeRequestVo,"客户已经停用");
            return new RespR(false,"客户已经停用");
        }
        if(StringUtils.isEmpty(tradeRequestVo.getStartTime()) || StringUtils.isEmpty(tradeRequestVo.getEndTime())){
            log.info("获取交易失败SelfCarServiceLImpl.getTradeList{},msg{}",tradeRequestVo,"时间为空");
            return new RespR(false,"时间为空");
        }
        if(DateUtil.getDistanceDays(tradeRequestVo.getEndTime(),tradeRequestVo.getStartTime())>90){
            log.info("获取交易失败SelfCarServiceLImpl.getTradeList{},msg{}",tradeRequestVo,"不能查询大于90天的交易");
            return new RespR(false,"不能查询大于90天的交易");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(tradeRequestVo.getCustomerName());
        customerRec.setCompanyName(tradeRequestVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() !=1 ){
            log.info("获取交易失败SelfCarServiceLImpl.getTradeList{},msg{}",tradeRequestVo,"客户未备案");
            return new RespR(false,"客户未备案");
        }
        CardTradeRequest cardTradeRequest = new CardTradeRequest();
        cardTradeRequest.setCardId(tradeRequestVo.getEtcId());
        cardTradeRequest.setCompanyNum(customerRec.getCompanyNum());
        cardTradeRequest.setStartExTime(DateUtil.dateFormate(tradeRequestVo.getStartTime()));
        cardTradeRequest.setEndExTime(DateUtil.dateFormate(tradeRequestVo.getEndTime()));
        cardTradeRequest.setTradeStatus(tradeRequestVo.getTradeStatus());
        RespR<CardTradeResponse> resp = selfCarInterface.cardTrade(cardTradeRequest);
        if(resp.getCode() == 1){
            log.info("获取交易失败SelfCarServiceLImpl.getTradeList{},msg{}",tradeRequestVo,resp.getMsg());
            return new RespR(false,resp.getMsg());
        }else{
            CardTradeResponse cardTradeResponse = resp.getData();
            List<CardTradeModel> result  = cardTradeResponse.getResult();
            //对不存在的tradeId入表
            if(result!=null && result.size()>0){
                customerCarRecMapper.selectByETCForLock(tradeRequestVo.getEtcId());
                for(CardTradeModel cardTradeModel:result){
                    SelfCarTrade selfCarTrade =  sellCarTradeMapper.selectByTradeId(cardTradeModel.getTradeId());
                    if(selfCarTrade == null){
                        selfCarTrade = new SelfCarTrade();
                        selfCarTrade.setInterType(tradeRequestVo.getInterType());
                        selfCarTrade.setCustomId(customerRec.getCustomerName());
                        selfCarTrade.setCompanyName(customerRec.getCompanyName());
                        selfCarTrade.setCompanyReferencenum(customerRec.getCompanyReferencenum());
                        selfCarTrade.setCompanyNum(customerRec.getCompanyNum());
                        selfCarTrade.setCardId(cardTradeModel.getCardId());
                        selfCarTrade.setTradeId(cardTradeModel.getTradeId());
                        selfCarTrade.setExTime(DateUtil.tTimeToDate(cardTradeModel.getExTime()));
                        selfCarTrade.setFee(cardTradeModel.getFee());
                        selfCarTrade.setStatus(tradeRequestVo.getTradeStatus());
                        sellCarTradeMapper.insert(selfCarTrade);
                    }else{
                        if(!selfCarTrade.getStatus().equals(tradeRequestVo.getTradeStatus()) ){
                            selfCarTrade.setStatus(tradeRequestVo.getTradeStatus());
                            selfCarTrade.setInterType(tradeRequestVo.getInterType());
                            sellCarTradeMapper.updateByTradeId(selfCarTrade);
                        }
                    }
                };
            }else {
                return new RespR(false,"未查得");
            }

            return new RespR(result);

        }
    }

    @Override
    @Transactional
    public RespR<String> applInvoice(InvoiceApplVo invoiceApplVo) {
        log.info("申请开票开始SelfCarServiceLImpl.applInvoice{}",invoiceApplVo);
        if(!customerService.checkMoneyEnough(invoiceApplVo.getCustomerName())){
            log.info("申请开票失败SelfCarServiceLImpl.applInvoice{},msg{}",invoiceApplVo,"余额不足");
            return new RespR(false,"余额不足");
        }
        Customer customer = customerMapper.selectByCustomerNameForLock(invoiceApplVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            log.info("申请开票失败SelfCarServiceLImpl.applInvoice{},msg{}",invoiceApplVo,"客户已经停用");
            return new RespR(false,"客户已经停用");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(invoiceApplVo.getCustomerName());
        customerRec.setCompanyName(invoiceApplVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() !=1 ){
            log.info("申请开票失败SelfCarServiceLImpl.applInvoice{},msg{}",invoiceApplVo,"客户未备案");
            return new RespR(false,"客户未备案");
        }
        CustomerCarRec customerCarRec = customerCarRecMapper.selectByETC(invoiceApplVo.getCardId());
        if(customerCarRec == null || customerCarRec.getRecStatus() != 1){
            log.info("申请开票失败SelfCarServiceLImpl.applInvoice{},msg{}",invoiceApplVo,"业务校验错误:该卡号不在该企业下绑定！\\n:803");
            return new RespR(false,"业务校验错误:该卡号不在该企业下绑定！\\n:803");
        }
        B2BInvoiceApplyRequest b2BInvoiceApplyRequest = new B2BInvoiceApplyRequest();
        b2BInvoiceApplyRequest.setCardId(invoiceApplVo.getCardId());
        b2BInvoiceApplyRequest.setCompanyNum(customerRec.getCompanyNum());
        List<B2BInvoiceApplyModel> b2BInvoiceApplyModels = new ArrayList<>();
        List<String> tradeIds = invoiceApplVo.getTradeIds();
        if(tradeIds == null || tradeIds.size()<=0){
            return new RespR(false,"没有交易Id");
        }
        for(String tradeId:tradeIds){
            B2BInvoiceApplyModel b2BInvoiceApplyModel = new B2BInvoiceApplyModel();
            b2BInvoiceApplyModel.setTradeId(tradeId);
            b2BInvoiceApplyModels.add(b2BInvoiceApplyModel);
        }
        b2BInvoiceApplyRequest.setTradeIdModel(b2BInvoiceApplyModels);
        RespR<B2BInvoiceApplyResponse> respR = selfCarInterface.b2BInvoiceApply(b2BInvoiceApplyRequest);
        if(respR.getCode()==1){
            log.info("申请开票失败SelfCarServiceLImpl.applInvoice{},msg{}",invoiceApplVo,respR.getMsg());
            return new RespR(false,respR.getMsg());
        }else{
            B2BInvoiceApplyResponse b2BInvoiceApplyResponse = respR.getData();
            SelfCarAppl selfCarAppl = new SelfCarAppl();
            selfCarAppl.setApplId(b2BInvoiceApplyResponse.getApplyId());
            selfCarAppl.setApplTime(b2BInvoiceApplyResponse.getReceiveTime());
            selfCarAppl.setCompanyNum(b2BInvoiceApplyRequest.getCompanyNum());
            selfCarAppl.setCustomId(customer.getCustomerName());
            selfCarAppl.setCardId(invoiceApplVo.getCardId());
            selfCarAppl.setInvoiceAmount(b2BInvoiceApplyResponse.getInvoiceAmount());
            selfCarAppl.setInvoiceCount(b2BInvoiceApplyResponse.getInvoiceCount());
            selfCarAppl.setInvoiceTaxAmount(b2BInvoiceApplyResponse.getInvoiceTaxAmount());
            selfCarAppl.setInterType(invoiceApplVo.getInterType());
            selfCarApplMapper.insert(selfCarAppl);
            double needFee = 0;
            //更新交易id申请开票时间，并将转态改为开票中
            for(String tradeId : tradeIds){
                SelfCarTrade selfCarTrade =  sellCarTradeMapper.selectByTradeId(tradeId);
                selfCarTrade.setStatus(2);
                selfCarTrade.setAclTime(DateUtil.getCurrentDateStr());
                selfCarTrade.setApplId(b2BInvoiceApplyResponse.getApplyId());
                selfCarTrade.setInterType(invoiceApplVo.getInterType());
                sellCarTradeMapper.updateByTradeId(selfCarTrade);
                //计费
                SelfCarCalculateInfor carCalculateInfor = new SelfCarCalculateInfor();
                carCalculateInfor.setCustomId(customerRec.getCustomerName());
                carCalculateInfor.setCompanyNum(customerRec.getCompanyNum());
                carCalculateInfor.setCompanyName(customerRec.getCompanyName());//公司名称
                carCalculateInfor.setCompanyLongName(customerRec.getCompanyBelongName());//主体名称
                carCalculateInfor.setCalTime(StringUtils.substring(selfCarTrade.getExTime(),0,7));
                carCalculateInfor.setFee(customer.getFee());
                carCalculateInfor.setCompanyReferencenum(customerRec.getCompanyReferencenum());
                carCalculateInfor.setEtcNum(selfCarTrade.getCardId());
                if(calculateInforMapper.selectByEtcAndTime(carCalculateInfor) == null){
                    needFee = MathUtil.add(needFee,carCalculateInfor.getFee());
                    calculateInforMapper.insert(carCalculateInfor);
                }
            }
            //客户统一扣费
            customer.setInvoiceTime(DateUtil.getCurrentDateStr());
            customer.setAccountBalance(MathUtil.sub(customer.getAccountBalance(),needFee));
            customerMapper.updateByPrimaryKeySelective(customer);
            //SelfCarTrade selfCarTrade =  sellCarTradeMapper.selectByTradeId(cardTradeModel.getTradeId());
            return new RespR(respR.getData());
        }

    }

    @Override
    public RespR<List<SelfCarInvoice>> getSelfCarInvoicesByApplyIds(ApplQueryInvVo applQueryInvVo) {
        log.info("申请开票开始SelfCarServiceLImpl.getSelfCarInvoicesByApplyIds{}",applQueryInvVo);
        if(!customerService.checkMoneyEnough(applQueryInvVo.getCustomerName())){
            return new RespR(false,"余额不足");
        }
        B2BInvoiceQueryByApplyRequest b2BInvoiceQueryRequest = new B2BInvoiceQueryByApplyRequest();
        Customer customer = customerMapper.selectByCustomerName(applQueryInvVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            return new RespR(false,"客户已经停用");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(applQueryInvVo.getCustomerName());
        customerRec.setCompanyName(applQueryInvVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() !=1 ){
            return new RespR(false,"客户未备案");
        }
        b2BInvoiceQueryRequest.setCardId(applQueryInvVo.getEtcNum());
        b2BInvoiceQueryRequest.setCompanyNum(customerRec.getCompanyNum());
        b2BInvoiceQueryRequest.setApplyId(applQueryInvVo.getApplId());
        RespR<B2BInvoiceQueryByApplyResponse> responseRespR = selfCarInterface.b2bContractQuery(b2BInvoiceQueryRequest);
        if(responseRespR.getCode() == 1){
            log.info("申请开票失败SelfCarServiceLImpl.getSelfCarInvoicesByApplyIds{},msg{}",applQueryInvVo,responseRespR.getMsg());
            return new RespR(false,responseRespR.getMsg());
        }else{
            B2BInvoiceQueryByApplyResponse response = responseRespR.getData();
            List<B2BInvoiceQueryByApplyModel> b2BInvoiceQueryByApplyModels = response.getResult();
            for(B2BInvoiceQueryByApplyModel b2BInvoiceQueryByApplyModel:b2BInvoiceQueryByApplyModels){
                SelfCarInvoice selfCarInvoice = new SelfCarInvoice();
                BeanUtils.copyProperties(b2BInvoiceQueryByApplyModel,selfCarInvoice);
                selfCarInvoice.setCompanyName(customerRec.getCompanyName());
                selfCarInvoice.setCompanyNum(customerRec.getCompanyNum());
                selfCarInvoice.setCustomerName(customerRec.getCustomerName());
                selfCarInvoice.setBuyerTaxpayerCode(customerRec.getCompanyReferencenum());
                selfCarInvoice.setExStation(b2BInvoiceQueryByApplyModel.getExStationName());
                selfCarInvoice.setEnStation(b2BInvoiceQueryByApplyModel.getEnStationName());
                selfCarInvoice.setInvoiceMakeTime(DateUtil.tTimeToDate(selfCarInvoice.getInvoiceMakeTime()));
                selfCarInvoice.setExTime(DateUtil.tTimeToDate(selfCarInvoice.getExTime()));
                if(selfCarInvoiceMapper.selectByTradeAndInvoiceMakeTime(selfCarInvoice)==null || selfCarInvoiceMapper.selectByTradeAndInvoiceMakeTime(selfCarInvoice).size()==0){
                    selfCarInvoiceMapper.insert(selfCarInvoice);
                }

            }
            //返回相应的发票
            return new RespR(b2BInvoiceQueryByApplyModels);
        }
    }
    @Override
    public void getSelfCarInvoicesByApplyIds(SelfCarAppl applQueryInvVo) {
        log.info("申请开票开始SelfCarServiceLImpl.getSelfCarInvoicesByApplyIds{}",applQueryInvVo);
        if(!customerService.checkMoneyEnough(applQueryInvVo.getCustomId())){
            log.info("余额不足{}",applQueryInvVo);
            return ;
        }
        CustomerRec customerRecParam = new CustomerRec();
        customerRecParam.setCustomerName(applQueryInvVo.getCustomId());
        customerRecParam.setCompanyName(applQueryInvVo.getCompanyNum());
        CustomerRec customerRec = customerRecMapper.selectByCustomerNameAndCompanyNum(customerRecParam);
        if(customerRec == null || customerRec.getRecStatus() != 1){
            log.info("备案信息不存在{}",applQueryInvVo);
            return;
        }
        B2BInvoiceQueryByApplyRequest b2BInvoiceQueryRequest = new B2BInvoiceQueryByApplyRequest();

        b2BInvoiceQueryRequest.setCardId(applQueryInvVo.getCardId());
        b2BInvoiceQueryRequest.setCompanyNum(applQueryInvVo.getCompanyNum());
        b2BInvoiceQueryRequest.setApplyId(applQueryInvVo.getApplId());
        RespR<B2BInvoiceQueryByApplyResponse> responseRespR = selfCarInterface.b2bContractQuery(b2BInvoiceQueryRequest);
        if(responseRespR.getCode() == 1){
            log.info("申请开票失败SelfCarServiceLImpl.getSelfCarInvoicesByApplyIds{},msg{}",applQueryInvVo,responseRespR.getMsg());
        }else{
            B2BInvoiceQueryByApplyResponse response = responseRespR.getData();
            List<B2BInvoiceQueryByApplyModel> b2BInvoiceQueryByApplyModels = response.getResult();
            for(B2BInvoiceQueryByApplyModel b2BInvoiceQueryByApplyModel:b2BInvoiceQueryByApplyModels){
                SelfCarInvoice selfCarInvoice = new SelfCarInvoice();
                BeanUtils.copyProperties(b2BInvoiceQueryByApplyModel,selfCarInvoice);
                selfCarInvoice.setCompanyName(customerRec.getCompanyName());
                selfCarInvoice.setCompanyNum(applQueryInvVo.getCompanyNum());
                selfCarInvoice.setCustomerName(applQueryInvVo.getCustomId());
                selfCarInvoice.setBuyerTaxpayerCode(customerRec.getCompanyReferencenum());
                selfCarInvoice.setExStation(b2BInvoiceQueryByApplyModel.getExStationName());
                selfCarInvoice.setEnStation(b2BInvoiceQueryByApplyModel.getEnStationName());
                selfCarInvoice.setInvoiceMakeTime(DateUtil.tTimeToDate(selfCarInvoice.getInvoiceMakeTime()));
                selfCarInvoice.setExTime(DateUtil.tTimeToDate(selfCarInvoice.getExTime()));
                if(selfCarInvoiceMapper.selectByTradeAndInvoiceMakeTime(selfCarInvoice)==null || selfCarInvoiceMapper.selectByTradeAndInvoiceMakeTime(selfCarInvoice).size()==0){
                    selfCarInvoiceMapper.insert(selfCarInvoice);
                }

            }
        }
    }
    @Override
    public RespR<List<B2BInvoiceListModel>> getSelfCarInvoicesByTime(SelfCarDueQueryVo selfCarDueQueryVo,CustomerRec customerRec) {
        log.info("申请开票开始SelfCarServiceLImpl.getSelfCarInvoicesByTime{}",selfCarDueQueryVo);
        B2BInvoiceQueryRequest b2BInvoiceQueryRequest = new B2BInvoiceQueryRequest();
        b2BInvoiceQueryRequest.setCompanyNum(customerRec.getCompanyNum());
        b2BInvoiceQueryRequest.setCardId(selfCarDueQueryVo.getCardId());
        b2BInvoiceQueryRequest.setStartInvoiceMakeTime(DateUtil.dateFormate(selfCarDueQueryVo.getStartTime()));
        b2BInvoiceQueryRequest.setEndInvoiceMakeTime(DateUtil.dateFormate(selfCarDueQueryVo.getEndTime()));

        RespR<B2BInvoiceQueryResponse> responseRespR = selfCarInterface.b2BInvoiceQuery(b2BInvoiceQueryRequest);
        if(responseRespR.getCode() == 1){
            log.info("申请开票失败SelfCarServiceLImpl.getSelfCarInvoicesByTime{},msg{}",selfCarDueQueryVo,responseRespR.getMsg());
            return new RespR(false,responseRespR.getMsg());
        }else{
            B2BInvoiceQueryResponse b2BInvoiceQueryResponse = responseRespR.getData();
            List<B2BInvoiceListModel> b2BInvoiceListModels = b2BInvoiceQueryResponse.getResult();
            if(b2BInvoiceListModels!=null && b2BInvoiceListModels.size()>0){
                for(B2BInvoiceListModel b2BInvoiceListModel:b2BInvoiceListModels){
                    SelfCarInvoice selfCarInvoice = new SelfCarInvoice();
                    BeanUtils.copyProperties(b2BInvoiceListModel,selfCarInvoice);
                    selfCarInvoice.setInvoiceMakeTime(DateUtil.tTimeToDate(selfCarInvoice.getInvoiceMakeTime()));
                    selfCarInvoice.setExTime(DateUtil.tTimeToDate(selfCarInvoice.getExTime()));
                    selfCarInvoice.setCustomerName(selfCarDueQueryVo.getCustomername());
                    selfCarInvoice.setCompanyName(customerRec.getCompanyName());
                    selfCarInvoice.setBuyerTaxpayerCode(customerRec.getCompanyReferencenum());
                    selfCarInvoice.setBuyerName(customerRec.getCompanyName());
                    selfCarInvoice.setCompanyNum(customerRec.getCompanyNum());
                    selfCarInvoice.setEnStation(b2BInvoiceListModel.getEnStationName());
                    selfCarInvoice.setExStation(b2BInvoiceListModel.getExStationName());
                    List<SelfCarInvoice> selfCarInvoice1 = selfCarInvoiceMapper.selectByCodeAndNum(selfCarInvoice);
                    if(selfCarInvoice1==null){
                        selfCarInvoiceMapper.insert(selfCarInvoice);
                    }
                }
            }else{
                return new RespR(false,"未查得");
            }
            return new RespR(b2BInvoiceListModels,responseRespR.getData().getResult().toString());
        }
    }

    @Override
    public RespR<List<B2bInvoicePackageModel>> getInvoicePackge(InvoicePackageVo invoicePackageVo) {
        log.info("申请开票开始SelfCarServiceLImpl.getInvoicePackge{}",invoicePackageVo);
        Customer customer = customerMapper.selectByCustomerName(invoicePackageVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            return new RespR(false,"客户已经停用");
        }
        if(!customerService.checkMoneyEnough(invoicePackageVo.getCustomerName())){
            return new RespR(false,"余额不足");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(invoicePackageVo.getCustomerName());
        customerRec.setCompanyName(invoicePackageVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() !=1 ){
            return new RespR(false,"客户未备案");
        }

        B2bInvoicePackageRequest b2bInvoicePackageRequest = new B2bInvoicePackageRequest();
        b2bInvoicePackageRequest.setCompanyNum(customerRec.getCompanyNum());
        b2bInvoicePackageRequest.setMakeMonth(invoicePackageVo.getMonth());
        RespR<B2bInvoicePackageResponse> b2bInvoicePackageResponseRespR = selfCarInterface.b2bInvoicePackage(b2bInvoicePackageRequest);
        if(b2bInvoicePackageResponseRespR.getCode() == 1){
            log.info("申请开票失败SelfCarServiceLImpl.getInvoicePackge{}，msg{}",invoicePackageVo,b2bInvoicePackageResponseRespR.getMsg());
            return new RespR(false,b2bInvoicePackageResponseRespR.getMsg());
        }else{

            if(b2bInvoicePackageResponseRespR.getData().getResult()==null || b2bInvoicePackageResponseRespR.getData().getResult().size()<=0){
                return new RespR (false,"未查得");
            }
            return new RespR(b2bInvoicePackageResponseRespR.getData().getResult());
        }
    }

    @Override
    public RespR getEtcInfo(EtcQueryVo etcQueryVo) {
        log.info("获取etc信息：SelfCarServiceLImpl.getEtcInfo{}",etcQueryVo);
        Customer customer = customerMapper.selectByCustomerName(etcQueryVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            return new RespR(false,"客户已经停用");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(etcQueryVo.getCustomerName());
        customerRec.setCompanyName(etcQueryVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() !=1 ){
            return new RespR(false,"客户未备案");
        }
        CardBindQueryListToBRequest cardQueryCardToBRequest = new CardBindQueryListToBRequest();
        cardQueryCardToBRequest.setCompanyNum(customerRec.getCompanyNum());//企业编号，必输
        cardQueryCardToBRequest.setCardId(etcQueryVo.getCardId());//卡户 ,非必输
        cardQueryCardToBRequest.setPlateNum(etcQueryVo.getPlateNum());//车牌号,非必输
        cardQueryCardToBRequest.setPalteColor(etcQueryVo.getPlateColor());//车牌颜色.非必输
        RespR<CardBindQueryListToBResponse> respR = selfCarInterface.cardBindQueryListToB(cardQueryCardToBRequest);

        if(customerRec.getInterType() == 0){
            if(respR.getCode() == 0 && (respR.getData().getResult()==null||respR.getData().getResult().size()<=0)){
                respR.setCode(2);
            }
            return respR;
        }
        if(respR.getCode() == 1){
            log.info("获取etc信息失败：SelfCarServiceLImpl.getEtcInfo{},msg{}",etcQueryVo,respR.getMsg());
            return new RespR(false,respR.getMsg());
        }
        return new RespR(respR.getData().getResult());
    }
    public RespR queryEtcInfo(EtcBindVo etcBindVo){
        log.info("获取etc信息：SelfCarServiceLImpl.queryEtcInfo{}",etcBindVo);
        StringBuffer res = new StringBuffer();
        if(etcBindVo== null || etcBindVo.getCards() ==null || etcBindVo.getCards().size()<=0){
            return new RespR(false,"无备案Etc");
        }
        Customer customer = customerMapper.selectByCustomerName(etcBindVo.getCustomerName());
        if(customer == null || customer.getAccstatus()!=0){
            res.append("#客户："+etcBindVo.getCustomerName()+"不存在或状态异常");
            return new RespR(false,res.toString());
        }
        CustomerRec customerRecP = new CustomerRec();
        customerRecP.setCustomerName(etcBindVo.getCustomerName());
        customerRecP.setCompanyName(etcBindVo.getCompanyName());
        CustomerRec customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRecP);
        if(customerRec == null || customerRec.getRecStatus()!=1){
            res.append("#客户："+etcBindVo.getCustomerName()+"企业:"+etcBindVo.getCompanyName()+"请先备案");
            return new RespR(false,res.toString());
        }
        CardQueryCardToBRequest cardQueryCardToBRequest = new CardQueryCardToBRequest();
        cardQueryCardToBRequest.setCompanyNum(customerRec.getCompanyNum());
        List<CardQueryCardToBModel> cardQueryCardToBModels = new ArrayList<>();
        etcBindVo.getCards().forEach(carVo -> {
            CardQueryCardToBModel cardQueryCardToBModel = new CardQueryCardToBModel();
            cardQueryCardToBModel.setPlateNum(carVo.getNum());
            cardQueryCardToBModel.setPlateColor(carVo.getColor());
            cardQueryCardToBModels.add(cardQueryCardToBModel);
        });
        cardQueryCardToBRequest.setVehicleList(cardQueryCardToBModels);
        RespR<CardQueryCardToBResponse> respR = selfCarInterface.cardQueryCardToB(cardQueryCardToBRequest);
        if(respR.getCode() == 1){
            log.info("获取etc信息失败：SelfCarServiceLImpl.queryEtcInfo{},msg{}",etcBindVo,respR.getMsg());
            return new RespR(false,respR.getMsg());
        }
        if(respR.getData().getResult() == null || respR.getData().getResult().size()<=0){
            log.info("获取etc信息失败：SelfCarServiceLImpl.queryEtcInfo{},msg{}",etcBindVo,"没有etc卡");
            return new RespR(false,"未查得");
        }
        return new RespR(respR.getData().getResult());

    }
    @Override
    public RespR customerEtcRec(EtcBindVo etcBindVo) {
        log.info("etc注册：SelfCarServiceLImpl.customerEtcRec{}",etcBindVo);
        if(etcBindVo== null || etcBindVo.getCards() ==null || etcBindVo.getCards().size()<=0){
            return new RespR(false,"无备案Etc");
        }
        StringBuffer res = new StringBuffer();
        Customer customer = customerMapper.selectByCustomerName(etcBindVo.getCustomerName());
        if(customer == null || customer.getAccstatus()!=0){
            res.append("#客户："+etcBindVo.getCustomerName()+"不存在或状态异常");
            return new RespR(false,res.toString());
        }
        CustomerRec customerRecP = new CustomerRec();
        customerRecP.setCustomerName(etcBindVo.getCustomerName());
        customerRecP.setCompanyName(etcBindVo.getCompanyName());
        CustomerRec customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRecP);
        if(customerRec == null || customerRec.getRecStatus()!=1){
            res.append("#客户："+etcBindVo.getCustomerName()+"企业:"+etcBindVo.getCompanyName()+"请先备案");
            return new RespR(false,res.toString());
        }
        return recEtc(etcBindVo,customerRec);
    }
    public RespR recEtc(EtcBindVo etcBindVo,CustomerRec customerRec){
        CardQueryCardToBRequest cardQueryCardToBRequest = new CardQueryCardToBRequest();
        cardQueryCardToBRequest.setCompanyNum(customerRec.getCompanyNum());
        List<CardBindingToBModel> cardBindingToBModels = new ArrayList<>();
        etcBindVo.getCards().forEach(carVo -> {
            CardBindingToBModel cardBindingToBModel = new CardBindingToBModel();
            cardBindingToBModel.setCardId(carVo.getEtcNum());
            cardBindingToBModels.add(cardBindingToBModel);
        });
        CardBindingToBRequest cardBindingToBRequest = new CardBindingToBRequest();
        cardBindingToBRequest.setMobile(etcBindVo.getMobile());
        cardBindingToBRequest.setCompanyNum(customerRec.getCompanyNum());
        cardBindingToBRequest.setCardIdList(cardBindingToBModels);
        RespR<CardBindingToBResponse> respR1 = selfCarInterface.cardBindingToB(cardBindingToBRequest);
        if(respR1.getCode() == 1){
            log.info("etc注册失败：SelfCarServiceLImpl.customerEtcRec{},msg{}",etcBindVo,respR1.getMsg());
            return new RespR(false,respR1.getMsg());
        }else{
            return new RespR(respR1.getData());
        }
    }

    public void updateETCRec(CustomerCarRec customerCarRec){
        //根据车牌号查询是新增还是更新
        log.info("检验成功入库参数{}",customerCarRec);
        CustomerCarRec customerCarRec1 = customerCarRecMapper.selectByETC(customerCarRec.getEtcNum());
        if (customerCarRec1 == null) {
            log.info("检验成功入库新增参数{}",customerCarRec);
            customerCarRecMapper.insert(customerCarRec);
        } else {
            log.info("检验成功入库更新参数{}",customerCarRec);
            customerCarRecMapper.updateETCByPrimaryKey(customerCarRec);
        }
    }
    @Override
    public RespR customerEtcRecValid(EtcValidVo etcValidVo) {
        log.info("etc注册校验：SelfCarServiceLImpl.customerEtcRecValid{}",etcValidVo);
        StringBuffer res = new StringBuffer();
        Customer customer = customerMapper.selectByCustomerName(etcValidVo.getCustomerName());
        if(customer == null || customer.getAccstatus()!=0){
            res.append("#客户："+etcValidVo.getCustomerName()+"不存在或状态异常");
            return new RespR(false,res.toString());
        }
        CustomerRec customerRecP = new CustomerRec();
        customerRecP.setCustomerName(etcValidVo.getCustomerName());
        customerRecP.setCompanyName(etcValidVo.getCompanyName());
        CustomerRec customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRecP);
        if(customerRec == null || customerRec.getRecStatus()!=1){
            res.append("#客户："+etcValidVo.getCustomerName()+"企业:"+etcValidVo.getCompanyName()+"请先备案");
            return new RespR(false,res.toString());
        }
        CardValidCodeToBRequest cardValidCodeToBRequest = new CardValidCodeToBRequest();
        cardValidCodeToBRequest.setCompanyNum(customerRec.getCompanyNum());
        cardValidCodeToBRequest.setMobile(etcValidVo.getMobile());
        cardValidCodeToBRequest.setValidCode(etcValidVo.getValidCode());
        RespR<CardValidCodeToBResponse> respR = selfCarInterface.cardValidCodeToB(cardValidCodeToBRequest);
        if(respR.getCode() == 1){
            log.info("etc注册校验失败：SelfCarServiceLImpl.customerEtcRecValid{},msg",etcValidVo,respR.getMsg());
            return new RespR(false,respR.getMsg());
        }else{
            /**
             * 绑定的etc卡入库
             */
            respR.getData().getResult().forEach(cardValidCodeToBModel -> {
                CustomerCarRec customerCarRec = new CustomerCarRec();
                customerCarRec.setBusinessType(CommonConstants.BUSSINESS_TYPE_SELFCAR);
                customerCarRec.setCompanyName(customerRec.getCompanyName());
                customerCarRec.setCustomerName(customerRec.getCustomerName());
                customerCarRec.setCompanyNum(customerRec.getCompanyNum());
                customerCarRec.setCarNum(cardValidCodeToBModel.getPlateNum());
                customerCarRec.setEtcNum(cardValidCodeToBModel.getCardId());
                customerCarRec.setCarColor(cardValidCodeToBModel.getPlateColor()+"");
                customerCarRec.setRecStatus(CommonConstants.REC_STATUS_SUC);
                customerCarRec.setSuccTime(DateUtil.getCurrentDateStr());
                customerCarRec.setCustPhone(etcValidVo.getMobile());
                customerCarRec.setInterType(etcValidVo.getInterType());
                updateETCRec(customerCarRec);
            });
            return new RespR(respR.getData().getResult().toString());
        }
    }
    @Override
    public RespR customerCarUnRec(List<CustomerCarRec> customerCarRecs) {
        log.info("etc解绑：SelfCarServiceLImpl.customerCarUnRec{}",customerCarRecs);
        if(customerCarRecs==null || customerCarRecs.size()<=0){
            return new RespR(false,"无解绑信息");
        }

        StringBuffer res = new StringBuffer();
        String resRespon = "";
        try {
            for(CustomerCarRec customerCarRec:customerCarRecs){
                CustomerRec customerRecP = new CustomerRec();
                customerRecP.setCustomerName(customerCarRec.getCustomerName());
                customerRecP.setCompanyName(customerCarRec.getCompanyName());
                CustomerRec customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRecP);
                if("0".equals(customerRec.getBussinessType())){
                    customerCarRec.setRecStatus(2);
                    customerCarRecMapper.updateETCByPrimaryKeySelective(customerCarRec);
                }
            }
            if(org.springframework.util.StringUtils.isEmpty(res.toString())){
                return new RespR(resRespon);
            }else{
                return new RespR(false,res.toString());
            }

        }catch (Exception e){
            return new RespR(false,e.getMessage());
        }

    }
}
